@@ -1,16 +1,29 @@
 package Net::DNS::RR::DS;
 
-# $Id: DS.pm,v 1.6 2002/12/20 10:20:21 olaf Exp $
+# $Id: DS.pm,v 1.12 2004/06/04 16:06:48 olaf Exp $
 
 
 use strict;
-use vars qw(@ISA $VERSION);
+use vars qw(@ISA $VERSION $_Babble);
 
 use Net::DNS;
 use Carp;
-use Digest::BubbleBabble qw( bubblebabble );
 
-$VERSION = do { my @r=(q$Revision: 1.6 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
+use Digest::SHA1  qw(sha1 sha1_hex );
+
+
+BEGIN {
+
+    $_Babble=0;    
+    $_Babble=1 unless (eval "require Digest::BubbleBabble; import Digest::BubbleBabble qw(bubblebabble)") ;
+
+}
+
+
+
+
+
+$VERSION = do { my @r=(q$Revision: 1.12 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
 my $debug=0;
 
 @ISA = qw(Net::DNS::RR);
@@ -45,8 +58,11 @@ sub new_from_string {
 	if ($string) {
 		$string =~ tr/()//d;
 		$string =~ s/;.*$//mg;
+		$string =~ s/\n//g;
 		my ($keytag,  $algorithm, $digtype, $digest) = 
-		    $string =~ /^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/;
+		    $string =~ /^\s*(\S+)\s+(\S+)\s+(\S+)\s+((\S+\s*)+)/;
+		# We allow spaces in the digest.
+		$digest=~s/\s//g;
 		$self->{"keytag"}=$keytag;
 		$self->{"algorithm"}=$algorithm;
 		$self->{"digtype"}=$digtype;
@@ -66,7 +82,7 @@ sub rdatastr {
 	    $rdatastr .= "  "  . "$self->{algorithm}";
 	    $rdatastr .= "  "  . "$self->{digtype}";
 	    $rdatastr .= "  "  . "$self->{digest}";
-	    $rdatastr .= " ; ".$self->babble;   
+	    $rdatastr .= " ; ".$self->babble if $_Babble;   
 	    }
 	else {
 	    $rdatastr = "; no data";
@@ -103,13 +119,17 @@ sub verify {
 
 sub babble {
     my $self=shift;
-    return bubblebabble(Digest=>$self->digestbin);
+    if ($_Babble){
+	return bubblebabble(Digest=>$self->digestbin);
+    }else{
+	return("");
+    }
 }
 
 
 sub create {
     my ($class, $keyrr ,%args) = @_;
-    use Digest::SHA1  qw(sha1 sha1_hex );
+
     my $self;
     $self->{"digtype"}=1;
 
@@ -196,7 +216,7 @@ $dsrr->print;
 
 The verify method will return 1 if the hash over the key provided in
 the argument matches the data in the $dsrr itself i.e. if the DS
-pointing to the KEY from the argument. It will return 0
+pointing to the DNSKEY from the argument. It will return 0
 otherwise.
 
 $dsrr->($keyrr);
@@ -233,6 +253,12 @@ Returns the digest as  binary material
 Returns the key tag of the key. (RFC2535 4.1.6)
 
 
+=head2 digtype
+
+   print "digest type" . " = " . $dsrr->digtype ."\n";
+
+Returns the digest type of the DS RR.
+
 =head2 babble
 
    print $dsrr->babble;
